
# Wrap parser into a library and call it from Python using ctypes.
# NOTE: keep this structure consistent with FileMapAndParameters.h

from ctypes import *
import sys
libpath = sys.path[0] + '/lib/libpars.so'
libpars = CDLL(libpath, mode=RTLD_GLOBAL)
#libpars = CDLL("lib/libpars.so", mode=RTLD_GLOBAL) original line of code

# define the data structures we are pulling from the libraries
class LambdaSet(Structure):
	_fields_ = [("min", c_double), # minimum value
	("max", c_double), # maximum value
	("n", c_int),  	   # number of steps between min and max
	("log", c_int),	   # 0 if linear steps; != 0 if log steps
	("base", c_double), # the log of the minimum value
	("mantissa", c_double)] # mantissa of the minum value

class D1(Structure):
	_fields_ = [("InputKernelFileName", c_char_p), 	# name of kernel file
		    ("InputMapFileName", c_char_p)]	# name of map file

class D3(Structure):
	_fields_ = [("InputKernelBaseDirName", c_char_p),	# base directory of kernel files
	("InputMapBaseDirName", c_char_p),		# base directory of map files
	("InputMaskBaseDirName", c_char_p),		# base directory of mask files
	("AvgKernelIndices", c_char_p),			# Avg Kernel specification line
	("nKernelMapFiles", c_int),			# number of kernel-map pairs
	("KernelFilenames", POINTER(c_char_p)),			# list of kernel file names
	("MapFilenames", POINTER(c_char_p)),			# list of map file names
	("MaskFilenames", POINTER(c_char_p))]			# list of mask file names

class InputParms(Structure): 
	_fields_ = [
	("d1", D1),		# 1d specific parameters
	("d3", D3),		# 3d specific parameters
	("rls", c_int),		# RLS inversion
	("ola", c_int),		# OLA inversion
	("lsqr", c_int),	# LSQR inversion
	("constrained", c_int),        # Constrained LSQR inversion
	("forward", c_int), 	# forward calculation
	("xt", c_int), 		# crosstalk term
	("outputBaseDirName", c_char_p),	# base directory for all output files
	("InputICovarFileName", c_char_p),	# Inverse covariance file name
	("InputRegFileName", c_char_p),		# Regularization file name
	("lambdaSet1", LambdaSet),		# Range of values for first lambda parameter
	("lambdaSet2", LambdaSet),		# Range of values for second lambda parameter
	("lambdaSet3", LambdaSet),		# Range of values for crosstalk term (nu)
	("OptionalFile", c_char_p), 		# Optional file
	("InitialGuessFile", c_char_p), 		# Initial guess for LSQR algorithm
	("scatterer", c_int)]				# Scatterer of interest for 3dv OLA
	

def parseInputFile(inputFileName, rls):
	libpars.getInputFromFile.argtypes = [c_char_p, POINTER(InputParms)]
	libpars.getInputFromFile.restype = c_int

	myparms = InputParms()
	if (rls):
		myparms.rls = 1
		myparms.ola = 0
	else:
		myparms.rls = 0
		myparms.ola = 1
	
	error = c_int
	error = libpars.getInputFromFile(inputFileName, byref(myparms))
	return myparms, error
	
def getProblemDimension():
	libpars.getProblemDimension.restype = c_int
	dim = libpars.getProblemDimension()
	return dim

	


