#! /usr/bin/env python
#
# Python script to validate input file for OLA_inversion

import subprocess
import os
import sys
import shutil
import messages, usage, filesysOps, parseOps, constants
import inputFileParser
import inversions

InstallDir = 'bin/'

# main part of script
def run():

	ExecutableName = 'inversion'
	
	ParseOnlyString = '-p'
	CopyAllInputFlag = '-c'
	DebugFlag = '-d'
	
	ErrorFree = True
	ParseOnly = False
	copyAll = False		# True if copying all input files to output base directory;
						# False if only copying input file to output base directory.
	# Default is running OLA
	Rls = False			# True if RLS inversion;
						# False if OLA inversion;
	RunWithDebug = False
	
	if len(sys.argv) == 1:
		usage.print_info()
		sys.exit(0)
	
	# parse any command line flags
	for i in sys.argv:
		if (i == ParseOnlyString):
			print messages.ParsingOnlyMsg
			ParseOnly = True
		elif (i == CopyAllInputFlag):
			print messages.CopyAllInputMsg
			copyAll = True
		elif (i == DebugFlag):
			RunWithDebug = True
			print messages.DebugFlag
		else: # assume this is the input file name
			inputFileName = i
	
	fullPathToInputFile = os.path.abspath(inputFileName)
	print 'Full path to input file is ' + fullPathToInputFile

	inParms, error = inputFileParser.parseInputFile(inputFileName, Rls)

	if (error):
		print "Error during parse, exiting."
		sys.exit()
		
	whichProblem = inputFileParser.getProblemDimension()
	# print "whichProblem ", whichProblem
		
	if (inParms.ola == 1) or (inParms.forward):
		Rls = False
	if (inParms.lsqr == 1):
		expectedMask = 1
	else:
		expectedMask = 0
		
	# process lambda, mu, sigma range parameters
	allOk = parseOps.ProcessRangeValues(inParms.lambdaSet1, inParms.lambdaSet2, inParms.forward, whichProblem)
	if (not allOk):
		ErrorFree = False
	
	if ((Rls) and (whichProblem != constants.Problem1d)):
		# check averaging kernel			
		if (inParms.rls == 1):
			# Averaging Kernel calculations
			allOk = parseOps.ProcessAvgKernel(inParms.d3.AvgKernelIndices)	
			if (not allOk):
				ErrorFree = False
	
	if ParseOnly:
		print messages.ParsingOnly
		sys.exit()
	
	# create new version of output directory as needed
	allOk, outBaseDir = parseOps.ProcessOutputBaseDirLine(inParms.outputBaseDirName)
	# verify files exist and copy to output directory as needed
	# need to modify this; no need to return filename
	if (whichProblem == constants.Problem1d):
		allOk, inDataFile = filesysOps.VerifyExists(inParms.d1.InputMapFileName, copyAll, outBaseDir, messages.dFileNoExist)
		if (not allOk):
			ErrorFree = False
		allOk, inKernel = filesysOps.VerifyExists(inParms.d1.InputKernelFileName, copyAll, outBaseDir, messages.KFileNoExist)
		if (not allOk):
			ErrorFree = False	
	else:
		#if (inParms.lsqr == 1):
		# verify base directories exist
		inKernelBaseDir = inParms.d3.InputKernelBaseDirName;
		if (not os.path.exists(inKernelBaseDir)):
			ErrorFree = False
			print messages.KernelBaseDirNoExist % inKernelBaseDir
		if (not inParms.forward):
			inMapBaseDir = inParms.d3.InputMapBaseDirName;
			if (not os.path.exists(inMapBaseDir)):
				ErrorFree = False
				print messages.MapBaseDirNoExist % inMapBaseDir
			if (expectedMask):
				inMaskBaseDir = inParms.d3.InputMaskBaseDirName;
				if (not os.path.exists(inMaskBaseDir)):
					ErrorFree = False
					print messages.MaskBaseDirNoExist % inMaskBaseDir
		# verify kernel, map (and mask) files exist
		expectedKernels = 1
		if (whichProblem == constants.Problem5d):   # if file is basically like 3d, but problem dim is 5d, then the only change is 3 kernels instead of 1
			expectedKernels = 3
					# if (len(ld) == expectedKernels+1+expectedMask):   # 2):
		for k in range(inParms.d3.nKernelMapFiles):
			# verify files exist in their base directories
			for i in range(expectedKernels):
				kernelFile = inParms.d3.KernelFilenames[k*expectedKernels + i]  # ld[i]
				kpath = os.path.join(inKernelBaseDir,kernelFile)
				allOk, kpath = filesysOps.VerifyExists(kpath, copyAll, outBaseDir, messages.KFileNoExist)
				if (not allOk):
					ErrorFree = False
			if (not inParms.forward):
				mapFile = inParms.d3.MapFilenames[k]
				mpath = os.path.join(inMapBaseDir,mapFile)	
				allOk, mpath = filesysOps.VerifyExists(mpath, copyAll, outBaseDir, messages.dFileNoExist)
				if (not allOk):
					ErrorFree = False		
				if (expectedMask):
					maskFile = inParms.d3.MaskFilenames[k]
					mpath = os.path.join(inMaskBaseDir,maskFile)	
					allOk, mpath = filesysOps.VerifyExists(mpath, copyAll, outBaseDir, messages.maskFileNoExist)
					if (not allOk):
						ErrorFree = False								

		if (inParms.OptionalFile):
			# input Target 1D function matrix file (f.fits)
			allOk, inTargetF = filesysOps.VerifyExists(inParms.OptionalFile, copyAll, outBaseDir, messages.TargetFNoExist)
			if (not allOk):
				ErrorFree = False
			
	#verify additional files exist
	if (inParms.InputICovarFileName == 'diagonal'):
		print 'using diagonal covariance matrix'
		inInverseNoiseCovar = inParms.InputICovarFileName
	else:
		if (not inParms.forward):
			allOk, inInverseNoiseCovar = filesysOps.VerifyExists(inParms.InputICovarFileName, copyAll, outBaseDir, messages.CIFileNoExist)
		else:
			allOk, inInverseNoiseCovar = filesysOps.VerifyExists(inParms.InputICovarFileName, copyAll, outBaseDir, messages.ModelFileNoExist)	
	if (not allOk):
		ErrorFree = False
	if (inParms.InputRegFileName):
		# input Regularization matrix file (R.fits)
		allOk, inReg = filesysOps.VerifyExists(inParms.InputRegFileName, copyAll, outBaseDir, messages.RFileNoExist)
		if (not allOk):
			ErrorFree = False
		
	if (not ErrorFree):
		print messages.ErrorsExiting
		sys.exit()
		
	L = ['not recognized', '1d', '2d', '3d_scalar', 'not recognized', '3d_vector']
	if (inParms.rls):
		moduleText = ' rls'
	if (inParms.ola):
		moduleText = ' ola'
	if (inParms.lsqr):
		if (inParms.constrained):
			moduleText = ' clsqr'
		else:
			moduleText = ' lsqr'
	if (inParms.forward):
		moduleText = ' forward'
	
		
	# echo all the parameters as currently understood
	if (whichProblem == constants.Problem1d):
		print "\n\n Solving Time Distance Inversion with parameters ... "
		print "                     Problem Dimension: ", L[whichProblem], moduleText
		print "(Output)                Base Directory: ", outBaseDir
		print "(Input)                       Map File: ", inDataFile
		print "(Input)          Noise Covariance File: ", inInverseNoiseCovar
		print "(Input)                    Kernel File: ", inKernel
		print "(Input)            Regularization File: ", inReg
	else:
		#if (modifier == 5): 
		#	whichProblem = constants.Problem5d
		print "\n\n Solving Time Distance Inversion with parameters ... "
		print "                      Problem Dimension: ", L[whichProblem], moduleText
		print "(Output)                 Base Directory: ", outBaseDir
		print "(Input) Base Directory for Kernel Files: ", inKernelBaseDir
		if (inParms.forward):
			print "(Input)                      Model File: ", inInverseNoiseCovar
		else:
			print "(Input)    Base Directory for Map Files: ", inMapBaseDir
			print "(Input)           Noise Covariance File: ", inInverseNoiseCovar
		if (inParms.rls):
			print "(Input)             Regularization File: ", inReg
		else: # OLA
			if (whichProblem >= constants.Problem3d): 
				print "(Input)                      Theta File: ", inReg
				if (not inParms.forward):
					print "(Input)                  Target 1D File: ", inTargetF
		
	# copy input file to output base directory
	shutil.copy2(fullPathToInputFile, outBaseDir)
	
	# if everything looks good, call the c-code executable  
	print messages.SolvingInversion

	originalDir = inParms.outputBaseDirName
	cmmd = InstallDir + ExecutableName
	li = [cmmd, '-f', fullPathToInputFile]
	if (outBaseDir != originalDir):
		li.append('-o')
		li.append(outBaseDir)
	if (RunWithDebug):
		li.append('-d')
	
	if (inParms.ola):
		error = inversions.olaRun(inParms, outBaseDir, RunWithDebug)
	elif (inParms.rls):
		error = inversions.rlsRun(inParms, outBaseDir, RunWithDebug)
	elif (inParms.lsqr):
		if (inParms.constrained):
			error = inversions.clsqrRun(inParms, outBaseDir, RunWithDebug)
		else:
			error = inversions.lsqrRun(inParms, outBaseDir, RunWithDebug)
	elif (inParms.forward):
		error = inversions.forwardRun(inParms, outBaseDir, RunWithDebug)
	else:
		print messages.UnrecognizedInversion
		error = 1
	
	if (error):
		print "Error during inversion, exiting."
		sys.exit()
	sys.exit()
