
import os
import constants, filesysOps, messages
	
# function to parse base output directory line
def ProcessOutputBaseDirLine(line):
	Bang = '!'
	MAX_DIRECTORY_VERION = 1000	
	allOk = True
	originalDir = line.strip() 
	originalDir = originalDir.rstrip(os.path.sep)
	overWrite = False
	if (originalDir.endswith(Bang)):
		overWrite = True
	vNumber = 2
	outBaseDir = originalDir
	# while directory exists and is not empty (list the content of the dir, if the returned list is empty, the dir is empty)
	while os.path.exists(outBaseDir) and os.listdir(outBaseDir) and (vNumber < MAX_DIRECTORY_VERION) and not overWrite:
		# directory is not empty; find the next empty directory.
		print 'Directory %s is not empty' %(outBaseDir)
		outBaseDir = originalDir + "_v%d" %(vNumber)
		print ' ... trying output dir %s' %(outBaseDir)
		vNumber += 1
	# base directory for output files
	# does directory exist?
	OBDExists = True
	if (not os.path.exists(outBaseDir)):
		# if not, can it be created?
		print 'Output base directory does not exist, attempting to create %s' % outBaseDir
		allOk, OBDExists = filesysOps.makeDirectory(outBaseDir)
	if (OBDExists):
		# is it writeable?
		if (not os.access(outBaseDir, os.W_OK)):  # W_OK is for writing; R_OK is for reading
			allOk = False
			print messages.OBDNoWrite %(line)
	return allOk, outBaseDir

	
# function to process Averaging Kernel line
def ProcessAvgKernel(line):
	allOk = True
	ld = line.split()
	if (len(ld) < 2):
		allOk = False
		print messages.AvgKerLineExpected %(line)
	Avg = ld[0]
	Kernel = ld[1]
	if (Avg.lower() != "avg"):
		allOk = False
		print messages.AvgStrExpected %(Avg)
	if (Kernel.lower() != "kernel"):
		allOk = False
		print messages.KernelStrExpected %(Kernel)	
	if (len(ld) > 2):
		try:
			depth = int(ld[2])
			if (depth == -1): #(int(ld[2]) == -1):
				# -1 indicates don't calculate averaging kernel and Cov
				# Avg Kernel -1
				if (len(ld) > 3):
					# print 'After -1 there is:', ld[3]
					allOk = False
					print messages.ExtraCharsAfterMinus1 %(line)	
			else:
				# verify positive integer numbers only after the characters
				for i in range(2, len(ld)):
					try:
						depth = int(ld[i])
						if (depth < 0):
							allOk = False
							print messages.ExpectedPositiveInteger %(depth)		
					except:
						allOk = False
						print messages.ExpectedPositiveInteger %(ld[i])		
		except:
			allOk = False
			print messages.ExpectedPositiveInteger %(ld[2])		
	return allOk
	
# function to process lambda/range values
# go through range specifications and validate three parameters (min, max, Nsteps)
# for each range,
# make sure Min < Max
# make sure NSteps > 0
# ProcessRangeValues(inParms.lambdaSet1, inParms.lambdaSet2, inParms.forward, whichProblem)
def ProcessRangeValues(lambdaSet1, lambdaSet2, forward, whichProblem):
	allOk = True
	lambdaMin = lambdaSet1.min # float(ld[0])
	lambdaMax = lambdaSet1.max # float(ld[1])
	lambdaN = lambdaSet1.n # float(ld[2])
	if (lambdaMin > lambdaMax):
		allOk = False
		print messages.RangeMinGreaterMax %(lambdaMin, lambdaMax)
	if (lambdaN <= 0):
		allOk = False
		print messages.RangeNLessEqZero %(lambdaN)
	if (whichProblem != constants.Problem1d) and (not forward):
		lambdaMin = lambdaSet2.min # float(ld[3])
		lambdaMax = lambdaSet2.max # float(ld[4])
		lambdaN = lambdaSet2.n # float(ld[5])
		if (lambdaMin > lambdaMax):
			allOk = False
			print messages.RangeMinGreaterMax %(lambdaMin, lambdaMax)
		if (lambdaN <= 0):
			allOk = False
			print messages.RangeNLessEqZero %(lambdaN)
	return allOk
