/*
 * SuperY.h
 *
 *  Created on: Jun 16, 2011
 *      Author: Brenda Javornik (e-mail: brendaj (at) cora.nwra.com)
 */

#ifndef SUPERY_H_
#define SUPERY_H_

	/*
	 * Format for super vector y (using z-major ordering):
	 * T[z=0, v=0]
	 * T[z=1, v=0]
	 * T[z=2, v=0]
	 * T[z=0, v=1]
	 *  ...
	 * T[z=Nz, v=2]
	 * Noise^2[a=0]
	 *  ...
	 * Noise^2[a=M]
	 */
	typedef struct {
		//dvec *y; // the real, packed super vector
		long     yLength;
		double   *yElements;
		int Nx;
		int Ny;
		int Nz;
		int Nm;
		int Nv;
		int noiseBase;
		int RC; // nRows * nCols = Nx * Ny
	} SuperVectorY;



#endif /* SUPERY_H_ */

	void InitSuperVectorY(SuperVectorY *y, MAP_DEPTH_MATRIX m);

	void RealToDoubleComplex(double *rData, int nelements, double complex *cData);

	BOOLEAN DoubleComplexToReal(double complex *cData, int nelements, double *rData);

	void unpackSuperY(const SuperVectorY *superY, MAP_DEPTH_MATRIX *AvgKern, MAP_DEPTH_MATRIX *noise, BOOLEAN waveSpace);

	BOOLEAN packSuperY(const MAP_DEPTH_MATRIX *AvgKern, const MAP_DEPTH_MATRIX *noise, SuperVectorY *superY, BOOLEAN realSpace);
