/*
 * FileMapAndParameters.c
 *
 *  Created on: May 12, 2010
 *      Author: Brenda Javornik (e-mail: brendaj (at) cora.nwra.com)
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "utilities.h"
#include "errorHandler.h"
#include "FileMapAndParameters.h"
#include "ProblemDimension.h"

/** freeInputParms frees dynamic memory held by InputParms object
 * @param[in] ip An InputParms object
 * @ingroup FileMapAndParameters
 */
void freeInputParms(InputParms ip) {
	int i;
	int dim;
	dim = getProblemDimension();

	if (dim >= PDC.D2) {
		int vdim = 1;
		int v;
		if (dim == PDC.D5) vdim = 3;
		for (i = 0; i < ip.d3.nKernelMapFiles; i++) {
			for (v=0; v<vdim; v++) {
				td_free(ip.d3.KernelFilenames[i*vdim + v], strlen(ip.d3.KernelFilenames[i*vdim + v])+1);
			}
			td_free(ip.d3.MapFilenames[i], strlen(ip.d3.MapFilenames[i])+1);
			if (ip.lsqr || ip.constrained) {
				// free masks as well
				td_free(ip.d3.MaskFilenames[i], strlen(ip.d3.MaskFilenames[i])+1);
			}
		}
		int size = ip.d3.nKernelMapFiles * sizeof(char*);
		td_free(ip.d3.KernelFilenames, size * vdim);
		td_free(ip.d3.MapFilenames, size);
		td_free(ip.d3.InputKernelBaseDirName, strlen(ip.d3.InputKernelBaseDirName)+1);
		td_free(ip.d3.InputMapBaseDirName, strlen(ip.d3.InputMapBaseDirName)+1);
		if (ip.lsqr || ip.constrained) {
			// free masks as well
			td_free(ip.d3.MaskFilenames, size);
			td_free(ip.d3.InputMaskBaseDirName, strlen(ip.d3.InputMaskBaseDirName)+1);
			// if there is an initial guess file, free it also
			if (ip.InitialGuessFile != NULL)
				td_free(ip.InitialGuessFile, strlen(ip.InitialGuessFile)+1);
		}

		if (!ip.rls && !ip.forward && dim >=PDC.D3) {
			td_free(ip.OptionalFile, strlen(ip.OptionalFile)+1);
		}

	} else {
		td_free(ip.d1.InputKernelFileName, strlen(ip.d1.InputKernelFileName)+1);
		td_free(ip.d1.InputMapFileName, strlen(ip.d1.InputMapFileName)+1);
	}
	td_free(ip.InputICovarFileName, strlen(ip.InputICovarFileName)+1);
	if (dim != PDC.D2)
		td_free(ip.InputRegFileName, strlen(ip.InputRegFileName)+1);
	td_free(ip.outputBaseDirName, strlen(ip.outputBaseDirName)+1);

}


