/*
 * List.c
 *
 *  Created on: May 26, 2010
 *      Author: Brenda Javornik (e-mail: brendaj (at) cora.nwra.com)
 */

#include <stddef.h>
#include "utilities.h"
#include "List.h"

/**
 * @defgroup SupportingModules Supporting Modules
 */

/**
 * @defgroup List List - List maintains a linked list of character strings as a stack.
 * @ingroup SupportingModules
 */

/**
 * Create a new list.
 * @return A new List.
 * @ingroup List
 */
List newList() {
	List l;

	l = (List) td_malloc(sizeof(Head));
	l->head = NULL;
	l->length = 0;
	return l;
}


/** Free the memory for a list of nodes.
 * @param[in,out] l A List to free.
 * @ingroup List
 */
void freeList(List l) {
	int i;
	Node *n, *p;

	n = l->head;
	i=0;
	while ((i<l->length) && (n!=NULL)) {
		p = n->next;
		td_free(n, sizeof(Node));
		n = p;
		i++;
	}
	td_free(l, sizeof(Head));
}

/** Insert item at the head of List
 * @param[in] item Pointer to a character string to store in the list
 * @param[in,out] l Add to this list
 * @ingroup List
 */
void insertList( char *item, List l) {
	Node *n;

	n = (Node *) td_malloc(sizeof(Node));
	n->content = item;
	n->next = l->head;
	l->head = n;
	l->length = l->length + 1;
}

/**
 * Return the number of nodes in a list.
 * @param[in] l A List
 * @ingroup List
 */
int getLengthList(List l) {
	if (l != NULL)
		return l->length;
	else
		return -1;
}

/**
 * Create an iterator to traverse a list.
 * @param[in] l A List to traverse
 * @return A ListIterator that allows access to the nodes in the List.
 * @ingroup List
 */
ListIterator newListIterator(List l) {
	ListIterator li;
	li = &(l->head);
	return li;
}

/**
 * Get the next character string stored in the List.
 * @param[in, out] li A ListIterator.
 * @return The character string next in the list.
 * @ingroup List
 */
char *listIteratorNext(ListIterator li) {
	Node *current;
	Node *p;
	char *content;

	p = *li;
	current = *li;
	if (current != NULL) {
		*li = p->next;
		content = current->content;
		td_free(current, sizeof(Node));
		return content;
	} else {
		return NULL;
	}
}

/** Check if iterator is at the end of the list.
 * @param[in] li A List Iterator.
 * @return TRUE if the List Iterator is at the end of its list.
 * @ingroup List
 */
int endOfList(ListIterator li) {
	if (*li == NULL)
		return TRUE;
	else
		return FALSE;
}
