/*
 * errorHandler.c
 *
 *  Created on: Apr 30, 2010
 *      Author: Brenda Javornik (e-mail: brendaj (at) cora.nwra.com)
 */
#include <stdio.h>
#include <stdlib.h>

#include "errorHandler.h"

/**
 * Report error
 * @param[in] msg String containing error message.
 * @param[in] location String containing the originating function name.
 * @param[in] severity ErrorSeverityType indicates the degree of concern for the error.
 * Corresponds to enum ErrorSeverity.
 */
void reportError(const char *msg, char *location, ErrorSeverityType severity) {
	switch (severity) {

	case FATAL:
		// report message to all listeners
		fprintf(stderr, "FATAL: %s: %s\n", msg, location);
		exit(EXIT_FAILURE);

	case ERROR:
		fprintf(stderr, "ERROR: %s: %s\n", msg, location);
		break;

	case WARNING:
		fprintf(stderr, "WARNING: %s %s\n", msg, location);
		break;
	case INFO:
		// report message to all listeners
		fprintf(stderr, "INFO: %s %s\n", msg, location);
		break;

	default:
		fprintf(stderr, "%s: %s\n", msg, location);
		break;

	}
	return;
}

/**
 * Report error and include some numeric information in the message.
 * @param[in] msg String containing error message.
 * @param[in] value Numeric information for inclusion in the message.
 * @param[in] location String containing the originating function name.
 * @param[in] severity ErrorSeverityType indicates the degree of concern for the error.
 * Corresponds to enum ErrorSeverity.
 */
void reportErrorInt(const char *msg, int value, char *location, ErrorSeverityType severity) {
	char msg2[MAX_ERROR_MSG_LENGTH];
	snprintf(msg2, MAX_ERROR_MSG_LENGTH, "%s, %d", msg, value);
	reportError(msg2, location, severity);
}
