/*
 * errorHandler.h
 *
 *  Created on: Apr 30, 2010
 *      Author: Brenda Javornik (e-mail: brendaj (at) cora.nwra.com)
 */

#ifndef ERRORHANDLER_H_
#define ERRORHANDLER_H_

/**
 * Level of error severity.
 * FATAL reports message and halts execution of program.
 * ERROR reports message and results may be incorrect or corrupt, but program attempts to complete.
 * WARNING reports message, program continues to completion.
 * INFO debugging and informative messages.
 */

#define MAX_ERROR_MSG_LENGTH 160

//enum ErrorSeverity {FATAL, ERROR, WARNING, INFO};

/**
 * Severity of error
 */
typedef enum {FATAL, ERROR, WARNING, INFO} ErrorSeverityType;

void reportError(const char *msg, char *location, ErrorSeverityType severity);
void reportErrorInt(const char *msg, int value, char *location, ErrorSeverityType severity);

const static char OutOfMemory[] = "Out of Memory";
const static char IncompatibleMatrices[] = "Incompatible matrix dimensions, exiting";
const static char BadCodeVersions[] = "Incompatible version of libraries and inversion code.";
const static char BadCommandLineOption[] = "Unrecognized command-line parameter.";
const static char NoInputData[] = "No input data, exiting";

#endif /* ERRORHANDLER_H_ */
