/*
 * utilities.c
 *
 *  Created on: May 24, 2010
 *      Author: Brenda Javornik (e-mail: brendaj (at) cora.nwra.com)
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "utilities.h"
#include "errorHandler.h"
#include "CoreErrorMessages.h"



int endsWith(char *string, char *ending) {
	int stringLen, endingLen;

	stringLen = strlen(string);
	endingLen = strlen(ending);

	if (stringLen < endingLen)
		return FALSE;

	return !strncmp(&string[stringLen - endingLen], ending, endingLen);
}

int longestName(char *filenames[], int nFiles) {
	int i, max, len;

	// find the longest filename for memory allocation
	max = 0;
	for (i=0; i<nFiles; i++) {
		if ((len = strlen(filenames[i])) > max) {
			max = len;
		}
	}
	return len;
}

/**
 * Compare two string up to n characters. Comparison is done
 * with lowercase version of each string.
 * @param[in] str1 A string to compare.
 * @param[in] str2 Another string to compare.
 * @param[in] n maximum number of characters to compare.
 * @return TRUE if strings are equal; FALSE otherwise.
 */
int lowerStrNCmp(const char *str1, const char *str2, int n) {
	int i=0;
	int Equal = TRUE;
	while ((str1[i] != '\0') && (str2[i] != '\0') && (i<n) && (Equal)) {
		if (tolower(str1[i]) != tolower(str2[i])) {
			Equal = FALSE;
		}
		i += 1;
	}
	if (i != n) Equal = FALSE;
	return Equal;
}

static int parserIdx = -1;
static char *parserLine = NULL;
static int parserLineLength = -1;

/**
 *  stringTokenizer and nextToken work together to parse a character
 *  string into tokens, separated by spaces.
 *  stringTokenizer intializes variables for breaking a string into tokens.
 * @param[in] line The string to break into words.
 */
void stringTokenizer(char *line) {
	parserIdx = -1;
	parserLine = line;
	parserLineLength = strlen(line);
}

/**
 * stringTokenizer and nextToken work together to parse a character
 * string into tokens, separated by spaces.
 * nextToken finds the next set of non-white space characters, which is
 * considered a token.
 * @return The next token in the string or NULL if there are no more tokens.
 */
char *nextToken() {
	char *p = NULL;

	if (parserIdx < parserLineLength) {
		parserIdx += 1;

		while ((parserLine[parserIdx] == ' ') && (parserIdx < parserLineLength)) {
			parserIdx += 1;
		}
		if (parserIdx < parserLineLength) {
			p = &parserLine[parserIdx];
			// move the parserIdx past this token
			while ((parserLine[parserIdx] != ' ') && (parserIdx < parserLineLength)) {
				parserIdx += 1;
			}
		}
	}
	return p;
}

/*
 * append the filename to the basedir
 * assumes there is enough room in basedir for filename
 */
char *constructFileName(char *basedir, char *filename) {
	if (!endsWith(basedir, "/")) {
		strcat(basedir, "/");
	}
	return strcat(basedir, filename);
}

static long MemoryInUse = 0;

char *td_malloc(size_t size) {

	char *mem = NULL;
	MemoryInUse += size;

	mem = malloc(size);
	if (mem == NULL) reportError(OutOfMemory, "td_malloc", FATAL);
	return mem;
}



void td_free(void *mem, size_t size) {
	MemoryInUse -= size;
	free(mem);
}


long getMemoryReport() {
	return MemoryInUse;
}

char *getLibraryVersion() {
	return CODE_VERSION;
}
