/*
 * BaseDirectory.h
 *
 *  Created on: Sep 8, 2010
 *      Author: Brenda Javornik (e-mail: brendaj (at) cora.nwra.com)
 */

#ifndef BASEDIRECTORY_H_
#define BASEDIRECTORY_H_

#include "Constants.h"

/**
 * @defgroup BaseDirectory BaseDirectory - String manipulations to construct all output file names.
 * BaseDirectory operations reuse BaseDirectory.Name character string.
 * Offsets are maintained into this string of the base output directory and
 * the end of the base output filename.
 * @ingroup InversionData
 */

// BaseDirectory is singleton-like; a global, single instance of this structure (class).
// Really, this is fancy form of String, where a max length is maintained.

// /directory/mu_sigma_kx_ky_A.fits
//                           ^ debugLength
//                     ^ baseLength
//            ^ dirLength
typedef struct {
	char *Name;			///< Character string containing the base output directory. Will always end with a directory separator. Is changed and reused. @ingroup BaseDirectory
	int dirLength; 		///< length of base dir name; start writing parameters at this point. Changes to match Name;  @ingroup BaseDirectory
	int debugLength;	///< end of debug base file name.  @ingroup BaseDirectory
	int baseLength; 	///< end of base file name.  @ingroup BaseDirectory
	int maxLength; 		///< number of chars allocated for dirName, any filename cannot exceed this value. Cannot change this value once set. @ingroup BaseDirectory
	char *filenameBank[MAX_ALPHA]; ///< bank of strings for constructing output filenames.  @ingroup BaseDirectory
} BaseDirectory;

// set and get pointer to single instance of BaseDirectory
BaseDirectory *newBaseDirectory(char *directoryName);

void destroyBaseDirectory(BaseDirectory *bd);

char **constructOutputFilenames(BaseDirectory *baseDir, const char *UniqueFilename, int muStep, int sigmaStep, int NAlpha);

void addParmsToFilename(BaseDirectory *baseName, int value1, int value2, BOOLEAN debug);

void addParmsToFilename2(BaseDirectory *baseName, int *values, int nValues, BOOLEAN debug);

char *finishFilename(BaseDirectory *baseName, const char *UniqueFilename, BOOLEAN compress, BOOLEAN debug);

char **finishFilenames(BaseDirectory *baseDir, const char *UniqueFilename, int NAlpha, BOOLEAN compress, BOOLEAN debug);

char **finishFilenamesWExt(BaseDirectory *baseDir, const char *UniqueFilename, int NAlpha,
		BOOLEAN compress, BOOLEAN debug, const char *extension);

char **constructBFilenames(BaseDirectory *baseDir, const char *UniqueFilename, int nVectors);

char **constructDTildaFilenames(const BaseDirectory *baseDir, const char *UniqueFilename, int nFiles, int *fnameLength);

char **constructAvgCovFilenames(const BaseDirectory *baseDir, const char *UniqueFilename, int nFiles, int nAvgKernels, int nVectors, int *fnameLength);

void freeNFilenames(char **filenames, int nFiles, int fnameLength);

#endif /* BASEDIRECTORY_H_ */
