/*
 * Constants.h
 *
 *  Created on: Oct 14, 2010
 *      Author: Brenda Javornik (e-mail: brendaj (at) cora.nwra.com)
 */

#ifndef CONSTANTS_H_
#define CONSTANTS_H_

#include "FITSParameters.h"

#define MAX_ALPHA	MAX_FITS_PARMS	///> Maximum number of scatterers.

struct constants{
	 char *extension;
	 char *gzipExtension;
	 char *txtExtension;
	 char *TargetFilename;
	 char *AvgKernFilename;
	 char *AvgKernFilename1d;
	 char *ModelMuName;
	 char *ModelSigmaName;
	 char *ModelNuName;
	 char *ModelLambda1Name;
	 char *ModelLambda2Name;
	 char *ModelNoiseScaleName;
	 char *ModelLambdaComment;
	 char *ModelComment;
	 char *NoiseScaleComment;
	 char *QFilename;
	 char *WFilename;
	 char *BFilename;
	 char *CovFilename;
	 char *NoiseFilename;
	 char *FileFlag;
	 char *OutputFlag;
	 char *DebugFlag;
	 char *DTILDA_FILENAME;
	 char *KX_FILENAME;
	 char *KY_FILENAME;
	 char *LAMBDA_FILENAME;
	 char *UNDERV[MAX_ALPHA]; //*UNDERV[] = {"v_x", "v_y", "v_z"};
	 char *XYZ[MAX_ALPHA]; //*XYZ[] = {"x", "y", "z"};
	 char *DirectorySeparator;
	 char *WK0Filename;
	 char *AvgKernK0Filename;
	 char *QK0Filename;
	 char *NoiseK0Filename;
};

static const struct constants Constants = {
		"fits",      		// FITS file extension
		"fits.gz",			// gzip file extension
		"txt",				// debug matrix extension
		"T",				// TargetFilename
		"AvgKern",			// averaging kernel Filename
		"A",				// averaging kernel Filename for RLS 1d
		"mu",       		// name in FITS header for mu parameter
		"sigma_h",  		// name in FITS header for sigma_h parameter
		"nu",  				// name in FITS header for crosstalk term (nu) parameter
		"LAMBDA1", 			///< Name of the lambda1 parameter for FITS header.
		"LAMBDA2",			///< Name of the lambda2 parameter for FITS header.
		"nscale",           ///< Name in FITS header for noise scale (forward calc).
		"lambda value", 	// 	comment in FITS header for RLS parameters
		"parameter value",	// comment in FITS header for parameters
		"noise scale",	    // comment in FITS header for noise scale
		"q",				// name of output file for perturbation, q
		"w",				// name of output file for inversion weights, w
		"b",				// name of output file for RLS solution vector(s)
		"Cov",				// name of output file for error covariance for RLS 1d
		"Noise",			// name of output file for noise
		"-f",
		"-o",				// indicates output base dir is not empty, so use the following name
		"-d", 				// command line flag to turn on debug;
		"d~",
		"kx",
		"ky",
		"lambda_squared",
		{"v_x", "v_y", "v_z"},
		{"x", "y", "z"},
		"/",					// directory separator
		"w_k0",				// name of output file for inversion weights, w, when ignoring K=0 element of weights
		"AvgKern_k0",		// averaging kernel Filename when ignoring K=0 element of weights
		"q_k0",				// name of output file for perturbation, q, when ignoring K=0 element of weights
		"Noise_k0"			// name of output file for noise when ignoring K=0 element of weights
};

#define MAX_DIGITS_IN_MAP  5
#define TWO_PI_SQUARED ((2.0 * M_PI) * (2.0 * M_PI))

#endif /* CONSTANTS_H_ */
