/*
 * Model.h
 *
 *  Created on: Aug 18, 2010
 *      Author: Brenda Javornik (e-mail: brendaj (at) cora.nwra.com)
 */

#ifndef INPUTDATA_H_
#define INPUTDATA_H_

#include "matrix.h"
#include "MapDepthMatrix.h"

/** @defgroup InputData Input Data - Structure holds all initial data read from FITS files.
 * @ingroup InversionData
 * @{
 */

typedef enum {RLS, OLA, LSQR, CLSQR} InvType ;

/**
 * InputData contains all input data read from files.
 */
typedef struct {
	MAP_DEPTH_MATRIX K;	///< Kernel data
	MAP_DEPTH_MATRIX M;	///< travel-time map data
	MAP_DEPTH_MATRIX C; ///< covariance data
	MAP_DEPTH_MATRIX mask; ///< mask data for LSQR
	MAP_DEPTH_MATRIX IG; ///< initial guess data for LSQR
	MATRIX K1d;			///< Kernel data for RLS 1D problem
	MATRIX M1d;			///< travel-time map data for RLS 1D problem
	MATRIX C1d;			///< covariance data for RLS 1D problem
	MATRIX R;			///< Regularization matrix for RLS 1D problem
	MATRIX Theta;		///< Overlap matrix for OLA 3D (Z > 1) problem
//	MATRIX Targetf;		///< Averaging kernel target function for OLA 3D problem
	InvType invType;
} InputData;


InputData *newInputData();

void destroyInputData(InputData *inData);

/** @} */

#endif /* INPUTDATA_H_ */
