/*
 * Model.h
 *
 *  Created on: Aug 18, 2010
 *      Author: Brenda Javornik (e-mail: brendaj (at) cora.nwra.com)
 */

#ifndef MODEL_H_
#define MODEL_H_

#include "MapDepthMatrix.h"
#include "FITSParameters.h"
#include "BaseDirectory.h"

/**
 * Inversion Solution and accompanying data.
 */
typedef struct {
	MAP_DEPTH_MATRIX w;			///< Inversion weights for OLA problems. @ingroup Model
	MAP_DEPTH_MATRIX q;			///< Estimate of pertubation for OLA problems. @ingroup Model
	MAP_DEPTH_MATRIX AvgKern;	///< Full Averaging Kernel matrix. @ingroup Model
	MAP_DEPTH_MATRIX noise;		///< Full Error Covariance matrix. @ingroup Model
	MAP_DEPTH_MATRIX dTilde;	///< Full d~ matrix for RLS 3D-scalar and 3D-vector problems. @ingroup Model
	MAP_DEPTH_MATRIX bFull;		///< Full solution matrix for RLS 3D-scalar and 3D-vector problems. @ingroup Model
	MATRIX kxGridValues;		///< Values of kx, in frequency space for RLS inversions. @ingroup Model
	MATRIX kyGridValues;		///< Values of ky, in frequency space for RLS inversions. @ingroup Model
	BOOLEAN kxkyDone;			///< Flag indicating if kx, ky grid has been written to file. @ingroup Model
	MATRIX lambdaGridValues;	///< Values of lambda for RLS inversions. @ingroup Model

	BOOLEAN convergence;		///< Flag indicating a solution was not found. @ingroup Model

	BOOLEAN xt;					///< Flag to include crosstalk in OLA-based inversions. @ingroup Model

	MATRIX bVECTORS;  		///< Solution matrix for RLS 1D problem; contains solutions for each lambda value as a column vector. @ingroup Model
	MATRIX A1d;				///< Averaging kernel for RLS 1D problem. @ingroup Model
	MATRIX Cov1d;			///< Error covariance for RLS 1D problem. @ingroup Model

	double mu; 		///< Regularization parameter value (lambda1 for RLS); kept in two places: here and in parms.values. @ingroup Model
	double sigma;   ///< Regularization parameter value (lambda2 for RLS); kept in two places: here and in parms.values. @ingroup Model
	double nu;      ///< Crosstalk parameter value; kept in two places: here and in parms.values. @ingroup Model
	FITSParameters parms; ///< parameter names, values and comments for header information in output FITS files. @ingroup Model
} Model;

void initializeModel(Model *model, int X, int Y, int M, int Z);

void initializeOutputModel(Model *model, MAP_DEPTH_MATRIX kernelTdm);

void initializeForwardOutputModel(Model *model, MAP_DEPTH_MATRIX kernelTdm);

void setParameters(Model *model, double mu, double sigma);

void setParameters2(Model *model, double *values, int nValues);

void clearParameters(Model *model);

void saveW(Model *model, MATRIX w, int kx, int ky);

void saveB(Model *model, MATRIX b, int nStep);

void saveGridValues(Model *m, int r, int c, double kx, double ky, double lambda);

void calculateQ(Model *model, MAP_DEPTH_MATRIX tau);

void calculateDerivedValues(Model *model, MAP_DEPTH_MATRIX K, MAP_DEPTH_MATRIX tau, MAP_DEPTH_MATRIX Lambda,
		BOOLEAN DValueShift);

void writeQ(Model *model, BaseDirectory *baseName);

void writeW(Model *model, BaseDirectory *baseName);

void writeAvgKern(Model *model, BaseDirectory *baseName);

void writeBVectors1d(Model *model, BaseDirectory *baseName);

void writeAVectors1d(Model *model, BaseDirectory *baseName, double lambda);

void writeCovVectors1d(Model *model, BaseDirectory *baseName, double lambda);

void writeNoise(Model *model, BaseDirectory *baseName);

void writeDTildaValues3d(Model *model, BaseDirectory *baseName, BOOLEAN useFT);

void writeAvgKernValues3d(Model *model, BaseDirectory *baseName);

void writeGridValues3d(Model *model, BaseDirectory *baseName);

void writeBVectors3d(Model *model, BaseDirectory *baseName);

void freeModel(Model *model);

#endif /* MODEL_H_ */
