/*
 * TempData.h
 *
 *  Created on: Nov 2, 2010
 *      Author: Brenda Javornik (e-mail: brendaj (at) cora.nwra.com)
 */

#ifndef TEMPDATA_H_
#define TEMPDATA_H_

#include "MapDepthMatrix.h"

/** @defgroup TempData Temp Data - Structure holds intermediate data during inversion
 * @ingroup InversionData
 * @{
 */

/**
 * Structure hold temporary data during inversion calculation.
 */
typedef struct {
	MAP_DEPTH_MATRIX K2piCC;	///< Complex conjugate of Kernel data multiplied by (2pi)^2
	MAP_DEPTH_MATRIX *T;		///< Target function
	MAP_DEPTH_MATRIX CI;		///< Inverse of travel-time noise covariance.
	MAP_DEPTH_MATRIX L;			///< Cholesky factorization of noise covariance
	MAP_DEPTH_MATRIX LCC;		///< Complex conjugate of Cholesky factorization of noise covariance
	MATRIX m; 					///< solution vector for RLS 3d
	MATRIX A; 					///< averaging kernel for one wave number, RLS 3d
	MATRIX Cov; 				///< error covariance for one wave number, RLS 3d
	MATRIX dTilda; 				///< d~ for one wave number, RLS 3d
	MAP_DEPTH_MATRIX dTildeFwd;	///< noise-free maps for forward calculation
} TempData;

/** @} */

#endif /* TEMPDATA_H_ */
