%{
#include <stdio.h>
#include <string.h>

#include "myptypes.h"
#include "utilities.h"
#include "FileMapAndParameters.h"
#include "List.h"
#include "lambda.h"
#include "ProblemDimension.h"
#include "parserErrorMessages.h"
	
#define MYDEBUG 0
#define MAX_AVG_KERN_LINE 200
#define YYSTYPE char *
 
YYLTYPE yylloc; 
const char *expecting = MissingProblemDefn;

BOOLEAN definitely = FALSE;

InputParms *inputParms;

List kList = NULL;
List mList = NULL;
List mkList = NULL;

char AvgKernLine[MAX_AVG_KERN_LINE];	

int iflog = 0;
int seed = 0;


int yywrap()
{
        return 1;
} 
 
void PullKernelFileNames();
void ConvertFileNamesToList(BOOLEAN readMask,  BOOLEAN readMap, int nkernels);
void yyerror(const char *str);

%}

%token T_AVG T_KERN T_1D T_2D T_3D T_3D_V T_3D_VX T_3D_VY T_3D_VZ T_LSQR T_CLSQR T_OLA T_RLS T_FORWARD T_INTEGER T_REAL T_FILENAME T_LOG T_SEED EOL

%%

input_file: 
        rls_1d |
         rls_3d | 
         rls_3dv | 
         ola_2d | 
         ola_3d | 
         ola_3dv | 
         lsqr_2d |
         lsqr_3d |
         lsqr_3dv |
         forward_3d |
         forward_3dv;

        
rls_1d:
	rls_1d_spec out_base_dir map_file covar_file kern_file theta_file parm ending
	{
		inputParms->d1.InputMapFileName = td_malloc(strlen($3)+1);
		inputParms->d1.InputKernelFileName = td_malloc(strlen($5)+1);
		strncpy(inputParms->d1.InputMapFileName, $3, strlen($3)+1);
		strncpy(inputParms->d1.InputKernelFileName, $5, strlen($5)+1);
		
		LambdaSet *lsPtr;
		lsPtr = (LambdaSet *) $7;
		inputParms->lambdaSet1 = *lsPtr;
		freeLambdaSet(lsPtr);
	};  
	
	
rls_3d:
		rls_3d_spec everybody scalar_3d_body theta_file avg_kern loop_parms ending
		{};      
		
rls_3dv:
		rls_3dv_spec everybody vector_3d_body theta_file avg_kern loop_parms ending
		{};  
        
ola_2d:
		ola_2d_spec everybody scalar_3d_body loop_parms ending
		{};
		
ola_3d:
		ola_3d_spec everybody scalar_3d_body theta_file loop_parms EOL target_file ending
		{};
		
ola_3dv:
		ola_3dv_spec everybody vector_3d_body theta_file loop_parms3 EOL target_file ending
		{};
		
lsqr_2d:
		lsqr_2d_spec everybody mask_base_dir lsqr_2d_km_pairs covar_file initial_guess_file loop_parms ending
		{
		};
		
lsqr_3d:
		lsqr_3d_spec everybody mask_base_dir lsqr_2d_km_pairs covar_file theta_file initial_guess_file loop_parms EOL target_file ending
		{
		};
		
lsqr_3dv:
		lsqr_3dv_spec everybody mask_base_dir lsqr_3dv_km_pairs covar_file theta_file initial_guess_file loop_parms3 EOL target_file ending
		{
		};

forward_3d:
	//  NUMBER OF EXPECTED KERNELS GOES WITH DIMENSION PLUS, EACH LINE OF KERNEL FILE IS A MAP INDEX
	//  Noise amplitude files take the place of map files. 
	//  There is one amplitude file for each kernel. 
	//  Theta file has the same meaning as in inversions.
	//  covar_file is really the model data file.
	//  Looping parameters scale the noise amplitude; a zero loop parameter == no noise.
	forward_3d_spec everybody scalar_3d_body theta_file seed parm ending
	{
		LambdaSet *lsPtr;
		lsPtr = (LambdaSet *) $6;
		inputParms->lambdaSet1 = *lsPtr;
		freeLambdaSet(lsPtr);
		
		// record the seed
		if (seed) {
			lsPtr = (LambdaSet *) $5;
			inputParms->lambdaSet2 = *lsPtr;
			freeLambdaSet(lsPtr);
		} else {
			inputParms->lambdaSet2.min = 0;
			inputParms->lambdaSet2.max = 0;
			inputParms->lambdaSet2.n = 0;
			inputParms->lambdaSet2.log = 0;
		}
	};
	
forward_3dv:
	//  NUMBER OF EXPECTED KERNELS GOES WITH DIMENSION PLUS, EACH LINE OF KERNEL FILE IS A MAP INDEX
	// covar_file is really the model data file
	// forward_3dv_spec out_base_dir kern_base_dir kernels3 covar_file theta_file parm ending
	forward_3dv_spec everybody vector_3d_body theta_file seed parm ending
	{
		LambdaSet *lsPtr;
		lsPtr = (LambdaSet *) $6;
		inputParms->lambdaSet1 = *lsPtr;
		freeLambdaSet(lsPtr);
		
		// record the seed
		if (seed) {
			lsPtr = (LambdaSet *) $5;
			inputParms->lambdaSet2 = *lsPtr;
			freeLambdaSet(lsPtr);
		} else {
			inputParms->lambdaSet2.min = 0;
			inputParms->lambdaSet2.max = 0;
			inputParms->lambdaSet2.n = 0;
			inputParms->lambdaSet2.log = 0;
		}
	};
		
rls_1d_spec:
	T_1D rls_module EOL
	{
		setProblemDimension(PDC.D1);
		inputParms->lsqr = FALSE;
		inputParms->rls = TRUE;
		inputParms->ola = FALSE;
		inputParms->forward = FALSE;	
		expecting = MissingBaseDir;
	};
		
ola_2d_spec:
		T_2D ola_module EOL
		{
			setProblemDimension(PDC.D2);
			inputParms->lsqr = FALSE;
			inputParms->constrained = FALSE;
			inputParms->rls = FALSE;
			inputParms->ola = TRUE;
			inputParms->forward = FALSE;	
			expecting = MissingBaseDir;
		};
		
		
ola_3dv_spec:
		ola_3dv_scat ola_module EOL
		{
			setProblemDimension(PDC.D5);
			inputParms->lsqr = FALSE;
			inputParms->constrained = FALSE;
			inputParms->rls = FALSE; 
			inputParms->ola = TRUE;
			inputParms->forward = FALSE;	
			expecting = MissingBaseDir;
		};
	
rls_3d_spec: 
		T_3D rls_module EOL
		{
			setProblemDimension(PDC.D3);
			if (definitely) {
				inputParms->lsqr = FALSE;
				inputParms->constrained = FALSE;
				inputParms->rls = TRUE; 
				inputParms->ola = FALSE;
				inputParms->forward = FALSE;	
			}
			expecting = MissingBaseDir;
		};
		
ola_3d_spec: 
		T_3D ola_module EOL
		{
			setProblemDimension(PDC.D3);
			if (definitely) {
				inputParms->lsqr = FALSE;
				inputParms->constrained = FALSE;
				inputParms->rls = FALSE; 
				inputParms->ola = TRUE;
				inputParms->forward = FALSE;	
			}
			expecting = MissingBaseDir;
		};
		
rls_3dv_spec:
		T_3D_V rls_module EOL
		{
			setProblemDimension(PDC.D5);
			inputParms->lsqr = FALSE;
			inputParms->constrained = FALSE;
			inputParms->rls = TRUE; 
			inputParms->ola = FALSE;
			inputParms->forward = FALSE;	
			expecting = MissingBaseDir;
		};
	 
lsqr_2d_spec:
		T_2D which_lsqr EOL
		{
			setProblemDimension(PDC.D2);
			inputParms->lsqr = TRUE;
			inputParms->rls = FALSE;
			inputParms->ola = FALSE;
			inputParms->forward = FALSE;	
			inputParms->InitialGuessFile = NULL;
			expecting = MissingBaseDir;
			
		};
		
lsqr_3d_spec:
		T_3D which_lsqr EOL
		{
			setProblemDimension(PDC.D3);
			inputParms->lsqr = TRUE;
			inputParms->rls = FALSE;
			inputParms->ola = FALSE;
			inputParms->forward = FALSE;	
			inputParms->InitialGuessFile = NULL;
			expecting = MissingBaseDir;
		};
		
lsqr_3dv_spec:
		ola_3dv_scat which_lsqr EOL
		{
			if (MYDEBUG) printf("inside lsqr_3dv_spec\n");
			setProblemDimension(PDC.D5);
			inputParms->lsqr = TRUE;
			inputParms->rls = FALSE; 
			inputParms->ola = FALSE;
			inputParms->forward = FALSE;	
			inputParms->InitialGuessFile = NULL;		
			expecting = MissingBaseDir;
		};
		
which_lsqr:
        plain_lsqr | constrained_lsqr
		
plain_lsqr:
        T_LSQR
        {
        	inputParms->constrained = FALSE;
        };
        
constrained_lsqr:
		 T_CLSQR
		 {
			inputParms->constrained = TRUE;
		 };
		
forward_3d_spec:
		T_3D T_FORWARD EOL
		{
			if (MYDEBUG) printf("inside forward_3d_spec\n");
			setProblemDimension(PDC.D3);
			inputParms->lsqr = FALSE;
			inputParms->rls = FALSE; 
			inputParms->ola = FALSE;
			inputParms->constrained = FALSE;
			inputParms->forward = TRUE;
			expecting = MissingBaseDir;
			inputParms->OptionalFile = NULL;
			if (MYDEBUG) printf("leaving forward_3d_spec\n");
		};
		 
forward_3dv_spec:
		T_3D_V T_FORWARD EOL
		{
			if (MYDEBUG) printf("inside forward_3dv_spec\n");
			setProblemDimension(PDC.D5);
			inputParms->lsqr = FALSE;
			inputParms->rls = FALSE; 
			inputParms->ola = FALSE;
			inputParms->constrained = FALSE;
			inputParms->forward = TRUE;
			expecting = MissingBaseDir;
			inputParms->OptionalFile = NULL;
			if (MYDEBUG) printf("leaving forward_3dv_spec\n");
		};
		
scalar_3d_body:
		km_pairs covar_file;
		
vector_3d_body:
		k3m_pairs covar_file;
		
everybody:
	out_base_dir kern_base_dir map_base_dir;
		
ola_3dv_scat:
	ola_3dv_x | ola_3dv_y | ola_3dv_z;
	
ola_3dv_x:
	T_3D_VX
	{
		inputParms->scatterer = 0;
	};
	
ola_3dv_y:
	T_3D_VY
	{
		inputParms->scatterer = 1;
	};
	
ola_3dv_z:
	T_3D_VZ
	{
		inputParms->scatterer = 2;
	};
		 
ola_module:
		T_OLA { definitely = TRUE;};
		
rls_module:
		T_RLS { definitely = TRUE;};
		
avg_kern:
		avg_kern_keywords depths EOL
		{
			inputParms->d3.AvgKernelIndices = AvgKernLine;
			if (inputParms->rls) expecting = MissingLambdaRange;
			else expecting = MissingMuSigmaRange;
		};
		
avg_kern_keywords:
	T_AVG T_KERN
	{
		strcat(AvgKernLine, "Avg ");
		strcat(AvgKernLine, "Kernel");
	};
		
depths: 
	| depths depth;
	
depth:
	T_INTEGER 
	{
		strcat(AvgKernLine, " ");
		if (strlen($1) + strlen(AvgKernLine) + 1 > MAX_AVG_KERN_LINE) {
		  printf("ERROR, Averaging kernel specification is too long\n");
		  exit(1);
		}
		strcat(AvgKernLine, $1);
	};
        



		 
lsqr_2d_km_pairs:
	lsqr_2d_km_pair | lsqr_2d_km_pairs lsqr_2d_km_pair 
	{
		expecting = MissingCovarFile;
	};
	
lsqr_2d_km_pair:
	kern_file_name map_file_name mask_file_name EOL
	{
		if (kList == NULL) {
			kList = newList();
			mList = newList();
			mkList = newList();
		} 
		char *one;
		int len;
		len = strlen($1)+1;
		one = td_malloc(len);
		strncpy(one, $1, len);
		insertList((char *) one, kList);		

		len = strlen($2)+1;
		one = td_malloc(len);
		strncpy(one, $2, len);
		insertList((char *) one, mList);
		
		len = strlen($3)+1;
		one = td_malloc(len);
		strncpy(one, $3, len);
		insertList((char *) one, mkList);		

	};
	
lsqr_3dv_km_pairs:
	lsqr_3dv_km_pair | lsqr_3dv_km_pairs lsqr_3dv_km_pair 
	{
		expecting = MissingCovarFile;
	};
	
lsqr_3dv_km_pair:
	kern_file_name kern_file_name kern_file_name map_file_name mask_file_name EOL
	{
		if (kList == NULL) {
			kList = newList();
			mList = newList();
			mkList = newList();
		} 

		int len;
		char *one;
		
		len = strlen($1)+1;
		one = td_malloc(len);
		strncpy(one, $1, len);
		insertList((char *) one, kList);
		
		len = strlen($2)+1;
		one = td_malloc(len);
		strncpy(one, $2, len);
		insertList((char *) one, kList);
		
		len = strlen($3)+1;
		one = td_malloc(len);
		strncpy(one, $3, len);
		insertList((char *) one, kList);
		
		len = strlen($4)+1;
		one = td_malloc(len);
		strncpy(one, $4, len);
		insertList((char *) one, mList);
		
		len = strlen($5)+1;
		one = td_malloc(len);
		strncpy(one, $5, len);
		insertList((char *) one, mkList);
		
	};
	
km_pairs:
	| km_pairs km_pair
	{
	};
	
km_pair:
	kern_file_name map_file_name EOL
	{
		if (kList == NULL) {
			kList = newList();
			mList = newList();
		} 

		int len;
		char *one;
		
		len = strlen($1)+1;
		one = td_malloc(len);
		strncpy(one, $1, len);
		insertList((char *) one, kList);
		
		len = strlen($2)+1;
		one = td_malloc(len);
		strncpy(one, $2, len);
		insertList((char *) one, mList);
		
	};
	
k3m_pairs:
	| k3m_pairs k3m_pair
	{
	};
	
k3m_pair:
	kern_file_name kern_file_name kern_file_name map_file_name EOL
	{
		if (kList == NULL) {
			kList = newList();
			mList = newList();
		} 

		int len;
		char *one;
		
		len = strlen($1)+1;
		one = td_malloc(len);
		strncpy(one, $1, len);
		insertList((char *) one, kList);
		
		len = strlen($2)+1;
		one = td_malloc(len);
		strncpy(one, $2, len);
		insertList((char *) one, kList);
		
		len = strlen($3)+1;
		one = td_malloc(len);
		strncpy(one, $3, len);
		insertList((char *) one, kList);
		
		len = strlen($4)+1;
		one = td_malloc(len);
		strncpy(one, $4, len);
		insertList((char *) one, mList);
		
	};

/*
kernels3:
	| kernels3 kernel3
	
kernel3:
	kern_file_name kern_file_name kern_file_name EOL
	{

		if (kList == NULL) {
			kList = newList();
		} 
		
		int len;
		char *one;
		
		len = strlen($1)+1;
		one = td_malloc(len);
		strncpy(one, $1, len);
		insertList((char *) one, kList);
		
		len = strlen($2)+1;
		one = td_malloc(len);
		strncpy(one, $2, len);
		insertList((char *) one, kList);
		
		len = strlen($3)+1;
		one = td_malloc(len);
		strncpy(one, $3, len);
		insertList((char *) one, kList);
		
	};
*/

loop_parms:
	parm parm
	{

		LambdaSet *lsPtr;
		lsPtr = (LambdaSet *) $1;
		inputParms->lambdaSet1 = *lsPtr;
		freeLambdaSet(lsPtr);
		
		lsPtr = (LambdaSet *) $2;
		inputParms->lambdaSet2 = *lsPtr;
		freeLambdaSet(lsPtr);		

    	inputParms->xt = FALSE;  
		
		if ((getProblemDimension() >= PDC.D3) && ((inputParms->ola) || (inputParms->lsqr)))
			expecting = Missing1dTargetFunc;

	};
	
loop_parms3:
	loop_parms xt
	{

/*
		LambdaSet *lsPtr;
		lsPtr = (LambdaSet *) $1;
		inputParms->lambdaSet1 = *lsPtr;
		freeLambdaSet(lsPtr);
		
		lsPtr = (LambdaSet *) $2;
		inputParms->lambdaSet2 = *lsPtr;
		freeLambdaSet(lsPtr);		
*/
		
		if ((getProblemDimension() >= PDC.D3) && ((inputParms->ola) || (inputParms->lsqr)))
			expecting = Missing1dTargetFunc;

	};
	
xt:
    |	
    { 
    	inputParms->xt = FALSE;  
    	printf("setting xt false\n"); 
    } 
    parm
    {
        // if this is a valid parameter speck, then this is the signal to 
        // include crosstalk in the inversion calculation
        LambdaSet *lsPtr;
        lsPtr = (LambdaSet *) $2;
		inputParms->lambdaSet3 = *lsPtr;
		freeLambdaSet(lsPtr);	
        inputParms->xt = TRUE;
        printf("setting xt true\n");
    };

ending:
	| ending an_eol
	{

	};
	
an_eol:
	EOL
	
parm:
	number number step
	{
		
		LambdaSet *lsPtr;
		double min, max;
		sscanf($1, "%lf", &min);
		sscanf($2, "%lf", &max);

		if (iflog == 0) {
			int n;
		    sscanf($3, "%d", &n);
		    lsPtr = newLambdaSet(min, max, n, iflog);
		} else {
		    // the number of steps doesn't matter for log; newLambdaSet will 
		    // determine the number of steps
			lsPtr = newLambdaSet(min, max, 1, iflog);
			// reset the iflog flag
			iflog = 0;
		}
		    
		// return a pointer to this lambda set
		$$ = (char *) lsPtr;
	};
	
step:
	 logstep | T_INTEGER;
	
logstep: 
	 T_LOG
	 {
		 iflog = 1;
	 };
	 
seed:
	| seed_def;
	
seed_def:
	T_SEED T_INTEGER EOL 
	{
	    // stuff the seed into the min and max value of a lambda set

		LambdaSet *lsPtr;
		double min;
		if (sscanf($2, "%lf", &min) != 1) {
		    expecting = UnsignedIntForSeed;
		    yyerror(expecting);
		}

        // check that seed is an unsigned integer
        if ((min < 0) || ((min - (int) min) > 0)) {
		    expecting = UnsignedIntForSeed;
		    yyerror(expecting);
		}
		unsigned int seedVal;
		seedVal = (unsigned int) min;
        printf("found random number generator seed %u\n", seedVal);
		lsPtr = newLambdaSet(min, min, 1, 0);
		    
		// return a pointer to this lambda set
		$$ = (char *) lsPtr;
		seed = 1;
	};

number:
	T_REAL | T_INTEGER;

		
out_base_dir:
	T_FILENAME EOL
	{
		if (MYDEBUG) printf("Outbasedir: %s\n", $1);
		inputParms->outputBaseDirName = td_malloc(strlen(yylval)+1);
		strncpy(inputParms->outputBaseDirName, yylval, strlen(yylval)+1);
		if (getProblemDimension() >= PDC.D2)
			expecting = MissingKernBaseDir;
		else
			expecting = MissingMapFile;
	};
	
mask_base_dir:
	T_FILENAME EOL
	{
		int len;
		len = strlen(yylval)+1;
		inputParms->d3.InputMaskBaseDirName = td_malloc(len);
		strncpy(inputParms->d3.InputMaskBaseDirName, yylval, len);
		expecting = MissingKernMapPairs;
	};
	
kern_base_dir:
	T_FILENAME EOL
	{
		if (MYDEBUG) printf("Kerbasedir: %s\n", $1);
		int len;
		len = strlen(yylval)+1;
		inputParms->d3.InputKernelBaseDirName = td_malloc(len);
		strncpy(inputParms->d3.InputKernelBaseDirName, yylval, len);
   		//inputParms->d3.InputKernelBaseDirName = (char *) yylval;
		expecting = MissingMapBaseDir;
	};
	
map_base_dir:
	T_FILENAME EOL
	{
		if (MYDEBUG) printf("Mapbasedir: %s\n", $1);
		int len;
		len = strlen(yylval)+1;
		inputParms->d3.InputMapBaseDirName = td_malloc(len);
		strncpy(inputParms->d3.InputMapBaseDirName, yylval, len);
		//inputParms->d3.InputMapBaseDirName = (char *) yylval;
		if ((inputParms->lsqr))
			expecting = MissingMaskBaseDir;
		else
			expecting = MissingKernMapPairs;
	};
	
mask_file_name:
	T_FILENAME
	{
		//printf("MskFileName: %s\n", $1);
	};
	
kern_file_name:
	T_FILENAME
	{
		if (MYDEBUG) printf("KerFileName: %s\n", $1);
	};
	
kern_file:
	T_FILENAME EOL
	{
		if (MYDEBUG) printf("KerFileName: %s\n", $1);
		expecting = MissingRegFile;
	};
	
map_file_name:
	T_FILENAME
	{
		if (MYDEBUG) printf("MapFileName: %s\n", $1);
		if ((inputParms->lsqr))
			expecting = MissingMaskFile;
	};

	
map_file:
	T_FILENAME EOL
	{
		if (MYDEBUG) printf("MapFileName: %s\n", $1);
		expecting = MissingCovarFile;
	};
	
theta_file:
	T_FILENAME EOL
	{
		// inputParms->InputRegFileName = (char *) yylval;
		inputParms->InputRegFileName = td_malloc(strlen(yylval)+1); // (char *) yylval;
		strncpy(inputParms->InputRegFileName, yylval, strlen(yylval)+1); 
		int dim;
		dim = getProblemDimension();
		if (dim == PDC.D1)
			expecting = MissingLambdaRange;
		else {
			if (inputParms->rls) 
				expecting = MissingAvgKern;
			else if (inputParms->forward)
			    expecting = MissingSeedOrNoiseScale;
			else
				expecting = MissingMuSigmaRange;
		}
	};

target_file:
	T_FILENAME
	{
		int len;
		len = strlen(yylval)+1;
		inputParms->OptionalFile = td_malloc(len);
		strncpy(inputParms->OptionalFile, yylval, len);
		//inputParms->OptionalFile = (char *) yylval;
		//printf("target file: %s\n", $1);
	};
	
initial_guess_file:
	| T_FILENAME EOL
	{
		int len;
		len = strlen(yylval)+1;
		inputParms->InitialGuessFile = td_malloc(len);
		strncpy(inputParms->InitialGuessFile, yylval, len);
		//inputParms->InitialGuessFile = (char *) yylval;
		printf("initial guess file: %s\n", $1);
	};
		
covar_file:
	T_FILENAME EOL
	{
		if (MYDEBUG) printf("CovFile: %s\n", $1);
		inputParms->InputICovarFileName = td_malloc(strlen($1)+1);
		strncpy(inputParms->InputICovarFileName, $1, strlen($1)+1);
		
		int dim;
		dim = getProblemDimension();
		if (dim != PDC.D1) {
			BOOLEAN mask = FALSE;
			BOOLEAN map = TRUE;
				if (inputParms->lsqr) 
					mask = TRUE;
				int nKernels = 1;
				if (dim == PDC.D5)
					nKernels = 3;
				ConvertFileNamesToList(mask, map, nKernels);
		}

		if (dim == PDC.D1)
			expecting = MissingKernFile;
		if (dim >= PDC.D3)
			expecting = MissingThetaFile;
	};
%%

void ConvertFileNamesToList(BOOLEAN readMask, BOOLEAN readMap, int nkernels) {
	if (MYDEBUG) printf("inside ConvertFileNamesToList\n");
			
		// convert filenames from List to char **
		int nkmFiles = getLengthList(mList);
		if (nkmFiles <= 0) {
			yyerror("No kernel-map pairs found");
		} else {
			inputParms->d3.nKernelMapFiles = nkmFiles;
			inputParms->d3.KernelFilenames = (char **) td_malloc(nkmFiles * nkernels * sizeof(char*));
			inputParms->d3.MapFilenames = (char **) td_malloc(nkmFiles * sizeof(char*));
			ListIterator kli = newListIterator(kList);
			ListIterator mli = newListIterator(mList);
	
			int i;
			for (i=nkmFiles-1; i>=0; i--) {
				inputParms->d3.MapFilenames[i] = listIteratorNext(mli);
			}
			for (i=(nkmFiles*nkernels)-1; i>=0; i--) {
				inputParms->d3.KernelFilenames[i] = listIteratorNext(kli);
			}
			freeList(kList);
			freeList(mList);
	
			if (readMask) {
				inputParms->d3.MaskFilenames = (char **) td_malloc(nkmFiles * sizeof(char*));
				ListIterator mli = newListIterator(mkList);
				for (i=nkmFiles-1; i>=0; i--) {
					inputParms->d3.MaskFilenames[i] = listIteratorNext(mli);
				}
				freeList(mkList);
			}
		}
		if (MYDEBUG) printf("leaving ConvertFileNamesToList\n");
}

// one function for each kernels, maps and masks 
void PullKernelFileNames() {
	if (MYDEBUG) printf("inside PullKernelFileNames\n");
	
		int dim;
		dim = getProblemDimension();
		int nKernels = 1;
		if (dim == PDC.D5)
			nKernels = 3;
			
		// convert filenames from List to char **
		int nkmFiles = getLengthList(kList)/nKernels;
		if (nkmFiles <= 0) {
			yyerror("No kernel-map pairs found");
		} else {
			inputParms->d3.nKernelMapFiles = nkmFiles;
			inputParms->d3.KernelFilenames = (char **) td_malloc(nkmFiles * nKernels * sizeof(char*));
			ListIterator kli = newListIterator(kList);
	
			int i;
			for (i=(nkmFiles*nKernels)-1; i>=0; i--) {
				inputParms->d3.KernelFilenames[i] = listIteratorNext(kli);
			}
			freeList(kList);
		}
		if (MYDEBUG) printf("leaving PullKernelFileNames\n");
}


int ParseError = 0;


void yyerror(const char *str)
{
        fprintf(stderr,"Error: %s near line %d\n",expecting, yylloc.first_line+1);
        ParseError = 1;
}

extern FILE *yyin; 


// YYLTYPE yylloc = {1,1,1,1};


//main()
// return 1 for success; return 0 if error
int getInputFromFile(char *inputFilename, InputParms *iParms)
{

  FILE *fp=fopen(inputFilename,"r");
  if (fp > 0) printf("Input file opened.\n");
  else {
	printf("Cannot open input file.\n");
	ParseError = 1;
	return ParseError;
  }
        yyin=fp;
  
  		AvgKernLine[0] = '\0';
  		inputParms = iParms;
        yyparse();
        return ParseError;

} 
